using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;

namespace r2musings.Web.UI.WebControls
{
  /// <summary>
  ///   CheckBox that controls display of other controls client-side
  /// </summary>
  [ToolboxData("<{0}:DisplayToggleCheckBox runat=server></{0}:DisplayToggleCheckBox>")]
  public class DisplayToggleCheckBox: System.Web.UI.WebControls.CheckBox
  {
    /// <summary>
    ///   Default Constructor
    /// </summary>
    public DisplayToggleCheckBox() { }

    #region Fields/Properties

    /// <summary>
    ///   Client-side function for custom Display toggle 
    /// </summary>
    [Category("Misc"), Browsable(true), DefaultValue("")]
    [Description("The client-side function to control the display toggle")]
    public string ClientDisplayToggleFunction
    {
      get
      {
        object o = this.ViewState["ClientDisplayToggleFunction"];
        if (o != null)
        {
          return (string) o;
        }
        return string.Empty;
      }
      set
      {
        this.ViewState["ClientDisplayToggleFunction"] = value;
      }
    }

    /// <summary>
    ///   One or more Control IDs (comma delimited) to display when the checkbox is checked
    /// </summary>
    [Category("Misc"), Browsable(true), DefaultValue("")]
    [Description("One or more Control IDs (comma delimited) to display when the checkbox is checked")]
    public string DisplayWhenCheckedControlIDs
    {
      get
      {
        object checkedControlIds = this.ViewState["DisplayWhenCheckedControlIDs"];
        if (checkedControlIds != null)
        {
          return (string) checkedControlIds;
        }
        return string.Empty;
      }
      set
      {
        this.ViewState["DisplayWhenCheckedControlIDs"] = value;
      }
    }

    /// <summary>
    ///   Actual Controls to display when CheckBox is checked
    /// </summary>
    protected List<WebControl> _DisplayWhenCheckedControls;
    [Browsable(false), DefaultValue(null)]
    [Description("The list of Controls to display when checkbox is checked")]
    protected List<WebControl> DisplayWhenCheckedControls
    {
      get
      {
        if (_DisplayWhenCheckedControls == null)
        {
          _DisplayWhenCheckedControls = new List<WebControl>();

          // get an array of the controls from the user-supplied property value
          string[] checkedControlIds = DisplayWhenCheckedControlIDs.Split(new Char[] { ',', ':', ';' });

          // loop through ids and add to List<>
          foreach (string controlId in checkedControlIds)
          {
            // using "as" to make sure we have a null and not an exception
            WebControl control = WebControls_Utility.FindControlRecursive(Page, controlId.Trim()) as WebControl;

            // no need to add control to List<> if control is not a WebControl
            if (control != null)
              _DisplayWhenCheckedControls.Add(control);
          }
        }

        return _DisplayWhenCheckedControls;
      }
    }
    
    /// <summary>
    ///   One or more Control IDs (comma delimited) to display when the checkbox is NOT checked
    /// </summary>
    [Category("Misc"), Browsable(true), DefaultValue("")]
    [Description("One or more Control IDs (comma delimited) to display when the checkbox is NOT checked")]
    public string DisplayWhenNotCheckedControlIDs
    {
      get
      {
        object notCheckedControlIds = this.ViewState["DisplayWhenNotCheckedControlIDs"];
        if (notCheckedControlIds != null)
        {
          return (string) notCheckedControlIds;
        }
        return string.Empty;
      }
      set
      {
        this.ViewState["DisplayWhenNotCheckedControlIDs"] = value;
      }
    }

    /// <summary>
    ///   Actual Controls to display when the CheckBox is NOT checked
    /// </summary>
    protected List<WebControl> _DisplayWhenNotCheckedControls;
    [Browsable(false), DefaultValue(null)]
    [Description("The list of Controls to display when checkbox is NOT checked")]
    protected List<WebControl> DisplayWhenNotCheckedControls
    {
      get
      {
        if (_DisplayWhenNotCheckedControls == null)
        {
          // create the List
          _DisplayWhenNotCheckedControls = new List<WebControl>();

          // get an array of the controls from the user-supplied property value
          //   should be comma-delimited
          string[] notCheckedControls = DisplayWhenNotCheckedControlIDs.Split(new Char[] { ',', ':', ';' });

          // loop through ids and add control to list of controls
          foreach (string controlId in notCheckedControls)
          {
            WebControl control = WebControls_Utility.FindControlRecursive(Page, controlId.Trim()) as WebControl;

            // no need to add control to List<> if control is not a WebControl
            if (control != null)
              _DisplayWhenNotCheckedControls.Add(control);
          }
        }

        return _DisplayWhenNotCheckedControls;
      }
    }

    /// <summary>
    ///   Dynamically creates the OnClientClick from the Checked and NotChecked control lists
    /// </summary>
    [Browsable(false)]
    public virtual string OnClientClick
    {
      get
      {
        // if there is a ClientDisplayToggleFuntion specified, include it
        string clientClick = (!String.IsNullOrEmpty(this.ClientDisplayToggleFunction)) ?
          String.Format("{0}(event);", this.ClientDisplayToggleFunction) : String.Empty;
      
        return String.Format("onClickDisplayToggleCheckBox(event, '{0}', '{1}');{2}",
          WebControls_Utility.GetCommaDelimitedClientIds(DisplayWhenCheckedControls),
          WebControls_Utility.GetCommaDelimitedClientIds(DisplayWhenNotCheckedControls),
          clientClick);
      }
    }
    
    #endregion
    
    #region Overridden methods
    
    /// <summary>
    ///   Override base to register client-side script
    /// </summary>
    /// <param name="e">EventArgs</param>
    protected override void OnPreRender(EventArgs e)
    {
      base.OnPreRender(e);

      if (this.Page != null) 
      {
        // register the javascript if not already registered
        if (!Page.ClientScript.IsClientScriptIncludeRegistered("r2DisplayToggleCheckBox"))
        {
          string scriptUrl = Page.ClientScript.GetWebResourceUrl(this.GetType(),
            "r2musings.Web.UI.script.r2DisplayToggleCheckBox.js");

          // register script 
          Page.ClientScript.RegisterClientScriptInclude("r2DisplayToggleCheckBox", scriptUrl);
        }
      }
    }
    
    /// <summary>
    ///   Override base to add the OnClick
    /// </summary>
    /// <param name="writer"></param>
    protected override void Render(HtmlTextWriter writer)
    {
      // write the onclick for the client 
      writer.AddAttribute("onclick", this.OnClientClick);

      //// initialize the Checked and NotChecked controls
      ////   NOTE:  Setting the Visible property of the WebControl causes
      ////   the control to NOT be rendered on the page and 
      ////   breaks the javascript, so show/hide them via CSS styles
      ////   to synch to the supplied Checked and NotChecked controls
      foreach(WebControl control in DisplayWhenCheckedControls)
      {
        // initialize IsCheckedControl to visible
        control.Visible = true;
      
        // set style.display on client ONLY if not checked
        if (!this.Checked)
          control.Style.Add(HtmlTextWriterStyle.Display, "none");
      }

      foreach (WebControl control in DisplayWhenNotCheckedControls)
      {
        // initialize IsCheckedControl to visible
        control.Visible = true;

        // set style.display on client ONLY if checked  
        if (this.Checked)
          control.Style.Add(HtmlTextWriterStyle.Display, "none");
      }

      base.Render(writer);
    }
    
    #endregion
    
  }
}



